#!/usr/bin/env python
# Copyright 2008-2009 David D Lowe
# released under the GPLv3
"""pigment is a module containing the parent class for all Pigment objects"""

# if pigment doesn't have an installation directory, it should use
# /tmp/epidermis/pigments/type/codename/

# TODO: Pigment should handle installationDirectory better
# TODO: Pigment should handle being in temp dir better (using installationDirectory?)
# TODO: handle meat

import pygtk
pygtk.require("2.0")
import gtk
from gettext import gettext as _
from xml.dom import minidom
import os, sys
import subprocess
from glob import glob
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.insert(0,"..") # for developing purposes
from shell import SubprocessShell
import ep_exceptions
import handy
import const
from const import PIGMENT_TYPES
import managepigments


TEMP_DIR = "TEMP"

QUERY_OVERWRITE = 1
QUERY_OVERWRITE_DATA = 2

STATE_EMPTY = 0
STATE_INFO = 2
STATE_INSTALLED = 3
STATE_TEMP = 4
STATE_INCOMPLETE = 1
#STATE_EMPTY: this is the state that the pigment initializes in
#STATE_INFO: the pigment only contains essential meta-data, for example, pigments
# viewed in the repo browser are in this state, no attachments
#STATE_INSTALLED: the pigment has been installed in its installationDirectory,
# all necessary attachments are saved
#STATE_TEMP: the pigment has been extracted to a temporary directory,
# including attachments
#STATE_INCOMPLETE: the pigment doesn't contain enough meta-data to qualify
#  as STATE_INFO, but it is not empty"""

class PigmentType():
    """The class for the type of pigment, has the following
    attributes: humanName, humanPluralName, codeName, abbreviation,
    directoryName
    humanName: human readable and translatable capitalised name of the pigment
    type
    humanPluralName: similar to humanName, but in the plural
    This class is an "abstract" class, it's meant to be subclassed"""
    humanName = {"en":""}
    humanPluralName = {"en":""}
    codeName = ""
    abbreviation = ""
    directoryName = ""
    canGeneratePreview = False
    
    attachmentTypeHelp = "" # what the attachment(s) should be, will be displayed
        # by the creator if is_attachment_correct returns False
        # should be surrounded by _( ) for translation, this string is read by
        # the end-users
    
    def __init__(self):
        pass
    
    # these functions are not yet used
    # in future, I plan to let PigmentType and Pigment objects control
    # how they are displayed in a TreeView widget
##    def get_number_of_columns(self):
##        """Returns number of columns that the pigment type needs to represent
##        itself in a gtk.TreeView"""
##        return 0
##        
##    def create_cell_renderer(self, index=0):
##        """Returns a new gtk.CellRenderer suitable for the data in column index"""
##        return None
##    
##    def set_human_name(self, humanName, language="en"):
##        self.humanName[language] = humanName
    
    def get_human_name(self, language="en"):
        return self.humanName[language]
    
    def __str__(self):
        return self.codeName
    
    def get_Pigment(self):
        return Pigment



class Pigment():
    """This Pigment class is the parent class for all other Pigment objects, 
    it contains the skeleton necessary for all Pigments
    This class is an "abstract" class, it's meant to be subclassed"""
    
    def __init__(self):
        """Initilises Psigment object, including the humanName, codeName, copyright,
        author, authorContact, description, preview, listStore,
        installationDirectory and type attributes"""
        # TODO: add version
        self.humanName = {"en":""}
        self.codeName = "" #: moohoo
        self.copyright = "" # TODO: differentiate between copyright and license
        self.author = ""
        self.authorContact = ""
        self.description = {"en":""}
        self.preview = None # of type gtk.gdk.Pixbuf
        self.previewFilename = "" # just filename
        self.previewPath = "" # path and filename
        self.installationDirectory = "" # under which base directory the pigment is currently saved
                                        # for example, this could be MY_DATA_HOME
        self.type = PigmentType()
        self.attachments = [] # list of filenames, for example, a wallpaper would have an image attached
                              # please note that the order of attachments cannot be relied upon
        self._acceptedPigmentVersions = ["0.1"]
        self._thisPigmentVersion = "0.1"
    
    def __str__(self):
        return _("Pigment of type %(type)s named %(codeName)s" % {'type':self.type.codeName, 'codeName':self.codeName})
    
    def set_human_name(self, humanName, language="en"):
        """Set a human name for this pigment in specified language,
        by default, en"""
        if len(language) == 0:
            language = "en"
        self.humanName[language] = humanName
    
    def get_human_name(self, language="en"):
        """Get the human name for this pigment in specified language"""
        if not language in self.humanName.keys() and self.humanName.has_key(""):
            return self.humanName[""]
        return self.humanName[language]
    
    def set_description(self, description, language="en"):
        """Set a human readable description for this pigment in specified language,
        by default, en"""
        self.description[language] = description
    
    def get_description(self, language="en"):
        """Get the human readable description for this pigment in specified
        language""" 
        return self.description[language]
    
    def add_attachment(self, attachment):
        """Add a file or a directory that may be needed for the installation
        and activation of the pigment. When saved, the file or directory will
        be found under data in the pigment compressed archive.
        Please note that the order of attachments cannot be relied upon.
        Use this function instead of appending to self.attachments directly.
        If attachment is not an absolute path, the pigment will look for it
        automatically based on installationDirectory"""
        if not os.path.exists(attachment):
            raise(ep_exceptions.AttachmentNotFoundException(attachment, pigment=self))
        self.attachments.append(attachment)
    
    
    def get_state(self):
        """Returns the state of the pigment, a number corresponding to one of
        these constants:
        STATE_EMPTY: this is the state that the pigment initializes in
        STATE_INFO: the pigment only contains essential meta-data, for example, pigments
            viewed in the repo browser are in this state, no attachments
        STATE_INSTALLED: the pigment has been installed in its installationDirectory,
            all necessary attachments are saved
        STATE_TEMP: the pigment has been extracted to a temporary directory,
            including attachments
        STATE_INCOMPLETE: the pigment doesn't contain enough meta-data to qualify
            as STATE_INFO, but it is not empty"""
        # check to see if pigment's essential meta-data is completed
        # only skins are excluded from the preview check
        if self.humanName != {"en":""} and \
        self.codeName != "" and \
        self.copyright != "" and \
        self.author != "" and \
        self.description != {"en":""} and \
        (not self.preview is None or self.type.codeName == "skin") and \
        self.type != PigmentType():
            # pigment's meta-data is sufficient
            if self.installationDirectory == "" or self.installationDirectory is None:
                return STATE_INFO
            elif os.path.exists("/tmp/epidermis/pigments") and os.path.samefile(self.installationDirectory, "/tmp/epidermis/pigments"):
                return STATE_TEMP
            else:
                hh = self.installationDirectory.split("/")
                if len(hh) > 3:
                    if hh[1] == "tmp":
                        return STATE_TEMP
                return STATE_INSTALLED
        else:
            # pigment's meta-data is insufficient
            if self.humanName == {"en":""} and \
            self.codeName == "" and \
            self.copyright == "" and \
            self.author == "" and \
            self.authorContact == "" and \
            self.description == {"en":""} and \
            self.preview is None and \
            self.previewFilename == "" and \
            self.previewPath == "" and \
            self.installationDirectory == "" and \
            len(self.attachments) == 0:
                return STATE_EMPTY
            else:
                return STATE_INCOMPLETE
    
    def debug_get_state_str(self):
        state = self.get_state()
        if state == STATE_EMPTY:
            return "STATE_EMPTY"
        elif state == STATE_INFO:
            return "STATE_INFO"
        elif state == STATE_INSTALLED:
            return "STATE_INSTALLED"
        elif state == STATE_TEMP:
            return "STATE_TEMP"
        elif state == STATE_INCOMPLETE:
            return "STATE_INCOMPLETE"
        else:
            return "Unknown, number is: %d" % state
    
    def set_preview_file(self, filename):
        """Set the preview file. If an absolute path is given, the
        pigment will load it"""
        if os.path.isabs(os.path.expanduser(filename)):
            if not os.path.exists(filename):
                raise(ep_exceptions.PreviewFileNotFoundException(filename))
            self.previewFilename = filename.split( "/")[-1]
            self.previewPath = os.path.expanduser(filename)
            self.preview =  gtk.gdk.pixbuf_new_from_file_at_size(filename, \
                const.PREVIEW_WIDTH,const.PREVIEW_HEIGHT)
        else:
            self.previewFilename = filename.split("/")[-1]
            self.previewPath = ""
            self.preview = None
    
    def _give_preview_file(self):
        """Returns the absolute path to a preview file for this pigment.
        If the preview filename hasn't been set, but a Pixbuf has, this
        will save a temporary file named preview.png and return its path"""
        subprocess.call(["mkdir", "-p", "/tmp/epidermis/preview"])
        if len(self.previewFilename) > 0 and len(self.previewPath) == 0:
            self.preview.save("/tmp/epidermis/preview" + self.previewFilename, self.previewFilename.split(".")[-1].lower())
            return "/tmp/epidermis/preview/" + self.previewFilename
        elif len(self.previewFilename) == 0 and not self.preview is None:
            self.preview.save("/tmp/epidermis/preview/preview.png", "png")
            return "/tmp/epidermis/preview/preview.png"
        else:
            return self.previewPath
    
    def get_preview(self):
        """Returns a Pixbuf for the preview"""
        if not self.preview is None:
            return self.preview
        elif len(self.previewPath) > 0:
            self.preview =  gtk.gdk.pixbuf_new_from_file_at_size(self.previewPath, 100, 75)
            return self.preview
        elif len(self.previewFilename) > 0:
            file = os.path.join(self.installationDirectory, self.type.directoryName, self.codeName, self.previewFilename)
            if os.path.exists(file):
                self.preview = gtk.gdk.pixbuf_new_from_file_at_size(file, 100, 75)
                return self.preview
            
        else:
            return None
    
    def generate_preview(self):
        """Automatically generate a preview pixbuf based on attachments and save it to self.preview
        Returning None means the method was not successful, otherwise, the method will return
        the newly generated self.preview"""
        return None
    
    def _extract_in_temp(self, filePath, shell=None):
        """Extracts the downloaded pigment to /tmp/epidermis/pigments/TYPE/CODENAME
        Returns the directory where the pigment has been extracted to.
        filePath: the path of the downloaded .pigment (string)
        shell: the Shell object that should be used, defaults to subprocess shell"""
        if not os.path.exists(filePath):
            raise(ep_exceptions.FileNotFoundException(filePath))
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        fileName = filePath.split("/")[-1]
        if self.codeName == "":
            mytmp = os.path.join("/tmp/epidermis/pigments", self.type.directoryName, TEMP_DIR)
        else:
            mytmp = os.path.join("/tmp/epidermis/pigments", self.type.directoryName, self.codeName)
        shell.do(["rm", "-r", "-f", mytmp])
        shell.do(["mkdir", "-p","-m","777", mytmp])
        shell.do(["cp", filePath, mytmp])
        shell.do(["mv", os.path.join(mytmp, fileName), os.path.join(mytmp, "%s.tar.gz" % fileName[:-8])])
        ##shell.do(["gunzip", "%s.tar.gz" % fileName[:-8]], cwd=mytmp)
        shell.do(["tar", "-xzf", "%s.tar.gz" % fileName[:-8]], cwd=mytmp)
        shell.do(["rm", os.path.join(mytmp, "%s.tar.gz" % fileName[:-8])])
        shell.do(["mv"] + glob(os.path.join(mytmp, "information", "*")) + [mytmp])
        shell.do(["rm", "-r", os.path.join(mytmp, "information")])
        return mytmp
    
    def _rearrange_temp(self, dir, shell=None):
        """Moves temp dir from dir to /tmp/epidermis/pigments/TYPE/CODENAME
        If preview file is in dir, it is also moved
        If any attachments are in /tmp, they're moved and re-added to attachments using self.add_attachment
        Sets self.installationDirectory to /tmp/epidermis/pigments/
        Returns temp dir"""
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        if self.codeName == "":
            print >> sys.stderr, "pigment's codename should be filled, exiting _rearrange_temp"
            return None
        shell.do(["rm", "-r", "-f", os.path.join("/tmp/epidermis/pigments", self.type.directoryName, self.codeName)])
        shell.do(["mv", dir, os.path.join("/tmp/epidermis/pigments", self.type.directoryName, self.codeName)])
        justPath = os.path.dirname(self._give_preview_file())
        if len(justPath) > 0: # not all pigments have previews
            if justPath[-1] == "/": justPath = justPath[:-1]
            if justPath == os.path.join("/tmp/epidermis/pigments", self.type.directoryName, TEMP_DIR):
                self.set_preview_file(os.path.join("/tmp/epidermis/pigments", \
                    self.type.directoryName, self.codeName, self.previewFilename))
        
        oldAttachments = []
        oldAttachments.extend(self.attachments)
        for attachment in oldAttachments:
            justPath = os.path.dirname(attachment)
            if attachment[-1] == "/": attachment = attachment[:-1]
            justFile = os.path.basename(attachment)
            if justPath[-1] == "/": justPath = justPath[:-1]
            if justPath == os.path.join("/tmp/epidermis/pigments", self.type.directoryName, \
            TEMP_DIR):
                self.attachments.remove(attachment)
                self.add_attachment( os.path.join("/tmp/epidermis/pigments", self.type.directoryName, \
                    self.codeName, justFile) )
            elif justPath == os.path.join("/tmp/epidermis/pigments", self.type.directoryName, \
            TEMP_DIR, "data"):
                self.attachments.remove(attachment)
                self.add_attachment( os.path.join("/tmp/epidermis/pigments", self.type.directoryName, \
                    self.codeName, "data", justFile) )
        
        self.installationDirectory = "/tmp/epidermis/pigments" #TODO: handle temp better
        
        return os.path.join("/tmp/epidermis/pigments", self.type.directoryName, self.codeName)
    

    
    def install(self, installationDirectory, progressCallback=None, shell=None):
        """Installs Pigment, ie, copies the information to installationDirectory/type/codename
        and copies the data to the appropriate place.
        installationDirectory: which base directory should be used to install
        the pigment (for example, MY_DATA_HOME)
        progressCallback: a function which accept two argument: progress: a
        float varying between 0.0 and 1.0, and query: a integer defaulting to None.
        If the function returns True then the installation will be aborted.
        query can be QUERY_OVERWRITE, which means that this method is asking
        whether it can overwrite
        shell: the Shell object that should be used to call commands as root
        Returns whether the installation was successfull, True or False"""
        
        # use subprocess shell if not specified
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        # ignore progressCallback if not specified
        if progressCallback is None:
            progressCallback = lambda progress, query: False
        # quit when ordered to
        if progressCallback(0.0, None): return False
        # check some things
        if len(self.codeName) == 0:
            raise(Exception("self.codeName is empty"))
        # copy the file
        if os.path.exists(os.path.join(installationDirectory, self.type.directoryName, self.codeName, "pigment.xml")):
            if progressCallback(0.1, QUERY_OVERWRITE): return False
        shell.do(["mkdir", "-p", os.path.join(installationDirectory, self.type.directoryName, self.codeName)])
        subprocess.call(["rm", "-r", "-f", "/tmp/epidermis/installer"])
        textxml = self.information_xml().toxml()
        subprocess.call(["mkdir", "-p", "/tmp/epidermis/installer"])
        fh = open("/tmp/epidermis/installer/pigment.xml", "wt")
        fh.write(textxml)
        fh.close()
        shell.do(["mv", "/tmp/epidermis/installer/pigment.xml", os.path.join(installationDirectory, self.type.directoryName, self.codeName, "pigment.xml")])
        if progressCallback(0.5, None): return False
        # copy preview file
        previewFile = self._give_preview_file()
        if len(previewFile) > 0:
            shell.do(["cp", previewFile, os.path.join(installationDirectory, self.type.directoryName, self.codeName, previewFile.split("/")[-1])])
            shell.do(["chmod", "644", os.path.join(installationDirectory, self.type.directoryName, self.codeName, previewFile.split("/")[-1])])
        if progressCallback(0.8, None): return False
        # chown to user
        if os.path.expanduser(installationDirectory).split("/")[:1] == ["","home"]:
            shell.do(["chown", "%s:%s" % (const.USERNAME, const.USERNAME), "-R", os.path.join(installationDirectory, self.type.directoryName, self.codeName)])
        # set this object's installationDirectory attribute
        self.installationDirectory = installationDirectory
        return not progressCallback(1.0, None)
        
            
            
    
    def remove(self, progressCallback=None, shell=None):
        """Uninstalls pigment.
        progressCallback: a function which accept one argument: progress: a
        float varying between 0.0 and 1.0. If the function returns True then 
        the uninstallation will be aborted
        shell: the Shell object that should be used to call commands as root"""
        # use subprocess shell if not specified
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        # ignore progressCallback if not specified
        def ignore(progress=None): return False
        if progressCallback is None:
            progressCallback = ignore
        # quit when ordered to
        if progressCallback(0.0): return
        # removed the pigment information
        shell.do(["rm", "-r", os.path.join(self.installationDirectory, self.type.directoryName, self.codeName)])
        # refresh self.installationDirectory
        self.installationDirectory = ""
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        """Applies the pigment to current user, and possibly system-wide
        progressCallback: a function which accepts one argument: progress: a
        float varying between 0.0 and 1.0 . If the function returns True then
        the activation will be aborted
        shell: the Shell object that should be used to call commands as root"""
        if not progressCallback is None:
            progressCallback(1.0)
        pass
    
    def save(self, filePath, shell=None):
        """Saves the pigment to filename as .pigment archive"""
        # TODO: save needs to be more easily 'sub-classed'
        # for example, the gtk_ pigment saves with it metacity
        doc = self.information_xml()
        text = doc.toxml()
        
        subprocess.call(["rm", "-f", "-r", "/tmp/epidermis/save"])
        subprocess.call(["mkdir", "-p", "/tmp/epidermis/save/information", "/tmp/epidermis/save/data/"])
        fh = open("/tmp/epidermis/save/information/pigment.xml", "wt")
        fh.write(text)
        fh.close()
        
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        
        previewFile = self._give_preview_file()
        shell.do(["cp", previewFile, "/tmp/epidermis/save/information"])
        if previewFile[0:-len(previewFile.split("/")[-1])] == "/tmp/epidermis":
            shell.do(["rm", previewFile])
        
        ll = [l2.split("/")[-1] for l2 in glob("/tmp/epidermis/save/*")]
        for item in self.attachments:
            shell.do(["cp", "-r", item, "/tmp/epidermis/save/data/"])
        shell.do(["tar", "-czf", filePath]+ ll, cwd="/tmp/epidermis/save/")
        #shell.do(["tar", "-czf", filePath, "information"], cwd="/tmp/epidermis")
        
    
    def load(self, filePath):
        """Load the Pigment object from a pigment file (.pigment)"""
        dir = self._extract_in_temp(filePath)
        self.read(os.path.join(dir, "pigment.xml"))
        tmp = self._rearrange_temp(dir)
        self.installationDirectory = "/tmp/epidermis/pigments/"
    
    def read(self, filePath):
        """Load the Pigment object from an xml file (.xml)
        This function will look for the preview file and for attachments, if required
        NOTE: this function will look for attachments in places other than filePath, 
        #TODO: change this?"""
        if not os.path.exists(filePath):
            raise(ep_exceptions.FileNotFoundException(filePath))
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        try:
            pigmentVersion = get_first_text(root.getElementsByTagName("pigmentVersion")[0])
        except:
            print >> sys.stderr, "Invalid <pigmentVersion> element"
            raise(ep_exceptions.PigmentVersionNotAccepted(None, self._acceptedPigmentVersions))
        if pigmentVersion not in self._acceptedPigmentVersions:
            print >> sys.stderr, "Invalid <pigmentVersion> element"
            raise(ep_exceptions.PigmentVersionNotAccepted(pigmentVersion, self._acceptedPigmentVersions))
        self.type = managepigments.get_pigment_type(get_first_text(root.getElementsByTagName("type")[0]))()
        self.humanName = {}
        try:
            for nameitem in root.getElementsByTagName("name"):
                self.set_human_name(get_first_text(nameitem), nameitem.getAttribute("lang"))
            for desitem in root.getElementsByTagName("description"):
                self.set_description(get_first_text(desitem),desitem.getAttribute("lang"))
        except ep_exceptions.NoFirstTextError, ee:
            print >> sys.stderr, "NoFirstTextError in " + filePath
            raise(ee)
        for xmlCode, var in (("codename", "codeName"), ("copyright","copyright"), \
        ("author", "author")):
            try:
                setattr(self, var, get_first_text(root.getElementsByTagName(xmlCode)[0]))
            except ep_exceptions.NoFirstTextError, ee:
                if xmlCode == "codename":
                    raise(ee)
                else:
                    setattr(self, var, "")
            except IndexError, ee:
                print >> sys.stderr, "Error in " + filePath + " for " + xmlCode
                raise(ee)
        if len(root.getElementsByTagName("preview")) == 1:
            try:
                self.previewFilename = get_first_text(root.getElementsByTagName("preview")[0])
            except ep_exceptions.NoFirstTextError, ee:
                self.previewFilename = ""
                self.set_preview_file("")
            else:
                
                fileDir = filePath[:(len(filePath) - len(filePath.split("/")[-1]))]
                if os.path.exists(os.path.join(fileDir, self.previewFilename)):
                    self.set_preview_file(os.path.join(fileDir, self.previewFilename))
                else:
                    self.set_preview_file(self.previewFilename)
        else:
            self.set_preview_file("")
    
    def write(self, filePath, shell=None):
        """Write the XML file to filePath
        Doesn't save preview"""
        # TODO: fixme ? write doesn't save preview
        subprocess.call(["rm", "/tmp/epidermis/pigment.xml"])
        subprocess.call(["mkdir", "-p", "/tmp/epidermis"])
        fh = open("/tmp/epidermis/pigment.xml", "wt")
        fh.write(self.information_xml().toxml())
        fh.close()
        
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        
        shell.do(["mv", "/tmp/epidermis/pigment.xml", filePath])
    
    def information_xml(self):
        """Returns the XML document describing the pigment, this document is
        usually saved as pigment.xml"""
        doc = minidom.Document()
        root = doc.createElement("pigment")
        doc.appendChild(root)
        
        #root.setAttribute("xmnls:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        #root.setAttribute("xmlns", "http://epidermis.tuxfamily.org")
        #root.setAttribute("xsi:schemaLocation","http://epidermis.tuxfamily.org ../wallpaper.xsd" )
        
        pigmentVersion = doc.createElement("pigmentVersion")
        pigmentVersion.appendChild(doc.createTextNode(self._thisPigmentVersion))
        root.appendChild(pigmentVersion)
        
        for xmlName, bit in (("name", self.humanName), ("description",self.description)):
            for lang in bit:
                if bit[lang] not in ("", None):
                    xmlBit = doc.createElement(xmlName)
                    xmlBit.setAttribute("lang", lang)
                    tn = doc.createTextNode(bit[lang])
                    xmlBit.appendChild(tn)
                    root.appendChild(xmlBit)
            
        for xmlName, bit in (("type", self.type.codeName), ("codename", self.codeName),("copyright",self.copyright), \
        ("author",self.author), ("preview",self.previewFilename)):
            xmlBit = doc.createElement(xmlName)
            tn = doc.createTextNode(bit)
            xmlBit.appendChild(tn)
            root.appendChild(xmlBit)
        
        
        return doc
    
    def is_attachment_correct(self, attachmentIndex=None):
        """Return whether the attachment number attachmentIndex or all the attachments
        if None is passed are correct.
        For example, a Metacity pigment would verify that the attachment is a
        directory with a metacity-1 subdirectory
        It does not verify the location of the attachment"""
        if attachmentIndex is None:
            for att in self.attachments:
                if not os.path.exists(att):
                    return False
            return True
        else:
            if os.path.exists(self.attachments[attachmentIndex]):
                return True
            else:
                return False
    
    #TODO: is_activated
##    def is_activated(self):
##        """Returns whether this pigment is applied.
##        Note: To check if the effects have been activated regardless of Epidermis'
##        pigments, use is_meat_activated"""
##        return
    
    def summary(self):
        """Returns a string: summary of the Pigment object"""
        return "Pigment codename: " + self.codeName \
            + "\nhumanName: " + str(self.humanName) \
            + "\ncopyright: " + self.copyright \
            + "\nauthor: " + self.author \
            + "\nauthorContact: " + self.authorContact \
            + "\ndescription: " + str(self.description) \
            + "\npreviewFileName: " + self.previewFilename \
            + "\ntype.codeName: " + self.type.codeName \
            + "\nattachments: " + self.attachments
    
##    def convert_from_meat(self, meat, progressCallback=None, block=True):
##        """Automatically grabs information from 'meat'. Note: this won't install
##        the pigment
##        meat: the name of the file or directory of the installed customisation
##        progressCallback: callback function, similar to the one passed to install
##        block: if False, this method will spawn a new thread"""
##        progressCallback(1.0)
##        pass
    
##    def is_meat_installed(self):
##        """Returns whether the meat of this pigment or similar to it is installed"""
##        return
##    
##    def is_meat_activated(self):
##        """Returns whether the meat of this pigment or similar to it is activated"""
##        return
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the Pigment object will need a Shell running as root
        to be passed to some of its methods"""
        return False
    
    def does_uninstallation_require_root(self):
        """Returns whether the activation requires root's rights.
        If it does, the Pigment object will need a Shell running as root
        to be passed to the remove method"""
        # TODO: check permissions of files and attachments to determine
        # whether uninstallation requires root
        return True
    
    def match_filter(self, matchString, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if matchString.lower() in ii.lower():
                return True
        return False
    
##    def get_display_data(self, index=0):
##        # almostTODO: display data
##        """Returns the object of index to be added to a gtk.ListStore, this
##        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
##        be rendered using type.create_cell_renderer"""
##        return

get_first_text = handy.get_first_text


def main():
    print "don't run this file"


if __name__ == "__main__":
    main()
